/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
#ifndef FPSCOUNTER_H
#define FPSCOUNTER_H
#include "time.h"
#include "Font.h"
#include "Shaders.h"
#include "Common.h"
#include "glm/gtx/transform.hpp"
#include "glm/gtc/matrix_inverse.hpp"
#include <glm/gtc/type_ptr.hpp>

#define WRITEFPS2FILE 1
#define TIMETHRESHOLD 30
#define ENABLE_FPS    1
#define FPS_FILE      "./FPSCounter.txt"
#if WRITEFPS2FILE
#include <iostream>
#include <iomanip>
#include <fstream>
#endif

#ifdef _WIN32
#include <windows.h>
#endif

#ifdef _linux_
#include <sys/time.h>
#endif

using namespace glTF;

#define VERTSHADER "Resource/font2D.vert"
#define FRAGSHADER "Resource/font2D.frag"
#define FONTPATH "Resource/DejaVuSans.ttf"

class FPSCounter
{
public:
    void timeStamp();

    void gpuTimeStampStart();
    void gpuTimeStampEnd();
    void gpuTimeAccumulation();
    void gpuAccumulationTimeclear();

    void printFPS(glTFViewport* pViewpoit);
    bool loadFPSShader(ShaderProgram* pShaderProgram);

    FPSCounter()
    {
        pFont = new Font();
        pFont->loadFont(FONTPATH, 32);
        mFPS = 0;
        mDurationTime = 0;
        mGPUTime = 0;
        mCurrentTime = 0;
        mLastTime = 0;
#ifdef _WIN32
        mTotalTime = 0;
        QueryPerformanceFrequency(&f);
        dqFreq = (double)f.QuadPart;
#endif

#if WRITEFPS2FILE
        FPSFile.open(FPS_FILE);
#endif
    }
    ~FPSCounter()
    {
        if (pFont != NULL)
        {
            pFont->deleteFont();
            delete pFont;
        }
        glDeleteProgram(uiFPSProgram);
#if WRITEFPS2FILE
        FPSFile.close();
        timeCounter = 0;
#endif
    }

private:
    int getFPS();
    double getExecutionTime();
    double getGPUTime();
    Font* pFont;

    unsigned int uiFPSProgram;
#ifdef _WIN32
    double mLastTime;
    double mCurrentTime;
    double mDurationTime;
    double dqFreq;
    LARGE_INTEGER f;

    double mGPUDuration;
    double mGPUStartTime;
    double mGPUEndTime;
    double mGPUTime;
    double mTotalTime;
#else
#ifdef _linux_
    struct timeval mLastTime;
    struct timeval mCurrentTime;
    double mDurationTime;
    double mGPUDuration;
    struct timeval mGPUStartTime;
    struct timeval mGPUEndTime;
    double mGPUTime;
    double mTotalTime;
#else

    clock_t mLastTime;
    clock_t mCurrentTime;
    clock_t mDurationTime;
    clock_t mGPUDuration;
    clock_t mGPUStartTime;
    clock_t mGPUEndTime;
    double mGPUTime;
#endif
#endif

    int mFPS;

#if WRITEFPS2FILE
    ofstream FPSFile;
    int timeCounter;
#endif
};

#endif
