/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
#ifndef LOADSCENE_H
#define LOADSCENE_H
#define CALC_PARSEING_TIME 1
#include <string>
#include <vector>
#include <boost/foreach.hpp>
#include <boost/property_tree/json_parser.hpp>

#include "Common.h"

using namespace glTF;

class Parser
{
public:
    glTFHandle* getFileNameInJson(const std::string& jsonFile);
    bool releaseFileName();
    bool parseScene(class Scene* pscene);
    bool parseJsonFile();
    void setJsonInfo(const std::string& sbuffer);
    void setJsonInfo(const std::string& direct, const std::string& fileName);
    Parser();
    ~Parser();
private:
    bool readBuffers();
    bool parseNodes();
    bool parseMeshs();
    bool parseMaterials();
    bool parseCameras();
    bool parseLights();
    bool parseAttributes();
    bool parseTechniques();
    bool parseAnim();

    Skin* parseSkins(const std::string& skinId);
    bool parsePrimitive(const boost::property_tree::ptree& pTree, class Mesh* pMesh);
    bool parseMaterialProper(const boost::property_tree::ptree& pTree,
                             class Material* pMaterial,
                             const std::string& techniqueId);
    void parseNode(const std::string& id, class Node* pParentNode,
                   const glm::mat4& pmatrix);
    void parseBone(const std::string& id, Bone* pParentBone);
    int parseTechniqueProgram(const boost::property_tree::ptree& pInstanceTree,
                               const boost::property_tree::ptree& pParameterTree,
                               class Technique* pTechnique);

    bool parseTechniqueState(boost::property_tree::ptree& pInstanceTree,
                             class Technique* pTechnique);

    std::string parseChannel(const boost::property_tree::ptree& pTree);
    int parsePara(const boost::property_tree::ptree& pTree, Animation* pAnimation);
    bool inline isFoundPtree(boost::property_tree::ptree& pTree, const char* key)
    {
        if (pTree.find(key) != pTree.not_found())
        {
            return true;
        }
        return false;
    };
    boost::property_tree::ptree ptParse;
    class Scene* pScene;
    std::string json_file;
    std::string json_directory;
    std::string json_buffer;
    bool is_json_in_buffer;
#if CALC_PARSEING_TIME
    double mParseingTime;
    double dqFreq;
    double mParseingStartTime;
    double mParseingEndTime;
#endif
};

#endif