/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
#include "Texture.h"

Texture::Texture()
{
    mMipMapsGenerated = false;
}

Texture::~Texture()
{
    deleteTexture();
}

void Texture::greateEmptyTexture(int a_iWidth, int a_iHeight, GLenum format)
{
    glGenTextures(1, &mTexture);
    glBindTexture(GL_TEXTURE_2D, mTexture);
    if (format == GL_RGBA || format == GL_BGRA)
    {
        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, a_iWidth, a_iHeight, 0, format,
                     GL_UNSIGNED_BYTE, NULL);
    }

    else if (format == GL_RGB || format == GL_BGR)
    {
        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, a_iWidth, a_iHeight, 0, format,
                     GL_UNSIGNED_BYTE, NULL);
    }
    else
    {
        glTexImage2D(GL_TEXTURE_2D, 0, format, a_iWidth, a_iHeight, 0, format,
                     GL_UNSIGNED_BYTE, NULL);
    }

    glGenSamplers(1, &mSampler);
}

void Texture::createFromData(unsigned char* bData, int a_iWidth, int a_iHeight,
                             GLenum format, bool bGenerateMipMaps)
{

    glGenTextures(1, &mTexture);
    glBindTexture(GL_TEXTURE_2D, mTexture);

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

    if (format == GL_RGBA || format == GL_BGRA)
    {
        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, a_iWidth, a_iHeight, 0, format,
                     GL_UNSIGNED_BYTE, bData);
    }

    else if (format == GL_RGB || format == GL_BGR)
    {
        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, a_iWidth, a_iHeight, 0, format,
                     GL_UNSIGNED_BYTE, bData);
    }
    else
    {
        glTexImage2D(GL_TEXTURE_2D, 0, format, a_iWidth, a_iHeight, 0, format,
                     GL_UNSIGNED_BYTE, bData);
    }
    if (bGenerateMipMaps)
    {
        glGenerateMipmap(GL_TEXTURE_2D);
    }
    glGenSamplers(1, &mSampler);

    mPath = "";
    mMipMapsGenerated = bGenerateMipMaps;
}

bool Texture::loadTexture2DFromBuffer(unsigned char* buffer, int width,
                                      int height)
{
    GLenum format = GL_BGRA;

    createFromData(buffer, width, height, format, false);

    return true;
}

void Texture::setSamplerParameter(GLenum parameter, GLenum value)
{
    glSamplerParameteri(mSampler, parameter, value);
}

void Texture::setFiltering(int a_tfMagnification, int a_tfMinification)
{
    glBindSampler(0, mSampler);

    if (a_tfMagnification == TEXTURE_FILTER_MAG_NEAREST)
    {
        glSamplerParameteri(mSampler, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    }
    else if (a_tfMagnification == TEXTURE_FILTER_MAG_BILINEAR)
    {
        glSamplerParameteri(mSampler, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    }

    if (a_tfMinification == TEXTURE_FILTER_MIN_NEAREST)
    {
        glSamplerParameteri(mSampler, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    }
    else if (a_tfMinification == TEXTURE_FILTER_MIN_BILINEAR)
    {
        glSamplerParameteri(mSampler, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    }
    else if (a_tfMinification == TEXTURE_FILTER_MIN_NEAREST_MIPMAP)
    {
        glSamplerParameteri(mSampler, GL_TEXTURE_MIN_FILTER,
                            GL_NEAREST_MIPMAP_NEAREST);
    }
    else if (a_tfMinification == TEXTURE_FILTER_MIN_BILINEAR_MIPMAP)
    {
        glSamplerParameteri(mSampler, GL_TEXTURE_MIN_FILTER,
                            GL_LINEAR_MIPMAP_NEAREST);
    }
    else if (a_tfMinification == TEXTURE_FILTER_MIN_TRILINEAR)
    {
        glSamplerParameteri(mSampler, GL_TEXTURE_MIN_FILTER,
                            GL_LINEAR_MIPMAP_LINEAR);
    }

    mMinification = a_tfMinification;
    mMagnification = a_tfMagnification;
}

void Texture::bindTexture(int iTextureUnit)
{
    glActiveTexture(GL_TEXTURE0 + iTextureUnit);
    glBindTexture(GL_TEXTURE_2D, mTexture);
    glBindSampler(iTextureUnit, mSampler);
}

void Texture::deleteTexture()
{
    glDeleteSamplers(1, &mSampler);
    glDeleteTextures(1, &mTexture);
}
